#include "cpu_mon.hpp"

#include <chrono>
#include <fstream>
#include <sstream>
#include <thread>

CpuMon::CpuMon(uint8_t num_hilos, WINDOW *ventanaMonitorCPU, int lineas) {
  CpuMon::num_hilos = num_hilos;
  CpuMon::ventanaMonitorCPU = ventanaMonitorCPU;
  CpuMon::enFuncionamiento = true;

  uint8_t num_sectores = 1;

  while (lineas < (int)(CpuMon::num_hilos / num_sectores)) {
    num_sectores++;
  }

  CpuMon::num_sectores = num_sectores;
  CpuMon::columnas = (ui::g_maxCols / 2) / CpuMon::num_sectores;
}

void CpuMon::esperar(int segundos) {
  std::this_thread::sleep_for(std::chrono::seconds(segundos));
}

void CpuMon::setNum_sectores(uint8_t num_sectores) {
  CpuMon::num_sectores = num_sectores;
}

void CpuMon::iniciarCPU_MON() {
  std::vector<DatosCPU> entradas1;
  std::vector<DatosCPU> entradas2;

  std::thread dormir(CpuMon::esperar, 1);

  // captura 1
  leerEstadoCPU(entradas1);

  // 100ms pause
  std::this_thread::sleep_for(std::chrono::milliseconds(500));

  // captura 2
  leerEstadoCPU(entradas2);

  // Mostrar entrada
  obtenerUsoCPU(entradas1, entradas2);

  dormir.join();
  // Se necesita limpiar los vectores de entrada, arreglo chapuza llamando
  // recursivamente la función a si misma
  if (CpuMon::enFuncionamiento) {
    CpuMon::iniciarCPU_MON();
  }
}

void CpuMon::pararCPU_MON() { CpuMon::enFuncionamiento = false; }

void CpuMon::leerEstadoCPU(std::vector<DatosCPU> &entradas) {
  std::ifstream fileStat("/proc/stat");

  std::string linea;

  const std::string STR_CPU("cpu");
  const std::size_t LEN_STR_CPU = STR_CPU.size();
  const std::string STR_TOT("tot");

  while (std::getline(fileStat, linea)) {
    // línea de datos de cpu encontrada
    if (!linea.compare(0, LEN_STR_CPU, STR_CPU)) {
      std::istringstream ss(linea);

      // almacenar entrada
      entradas.emplace_back(DatosCPU());
      DatosCPU &entrada = entradas.back();

      // leer etiqueta de cpu
      ss >> entrada.cpu;
      // eliminar "cpu" de la etiqueta cuando es el número de un procesador
      if (entrada.cpu.size() > LEN_STR_CPU) {
        entrada.cpu.erase(0, LEN_STR_CPU);
        // reemplazar "cpu" con "tot" cuando son los valores totales
      } else {
        entrada.cpu = STR_TOT;
      }
      // leer tiempos
      for (int i = 0; i < NUM_ESTADOS_CPU; ++i) {
        ss >> entrada.tiempos[i];
      }
    }
  }
}

size_t CpuMon::getTiempoInactivo(const DatosCPU &e) {
  return e.tiempos[S_IDLE] + e.tiempos[S_IOWAIT];
}

size_t CpuMon::getTiempoActivo(const DatosCPU &e) {
  return e.tiempos[S_USER] + e.tiempos[S_NICE] + e.tiempos[S_SYSTEM] +
         e.tiempos[S_IRQ] + e.tiempos[S_SOFTIRQ] + e.tiempos[S_STEAL] +
         e.tiempos[S_GUEST] + e.tiempos[S_GUEST_NICE];
}

void CpuMon::obtenerUsoCPU(const std::vector<DatosCPU> &entradas1,
                           const std::vector<DatosCPU> &entradas2) {

  const size_t NUM_ENTRADAS = entradas1.size();

  std::vector<std::string> salida(CpuMon::num_hilos);

  // Comienza en 1 para saltarse el total
  for (uint8_t i = 1; i < NUM_ENTRADAS; ++i) {
    const DatosCPU &e1 = entradas1[i];
    const DatosCPU &e2 = entradas2[i];

    const float TIEMPO_ACTIVO =
        static_cast<float>(getTiempoActivo(e2) - getTiempoActivo(e1));
    const float TIEMPO_INACTIVO =
        static_cast<float>(getTiempoInactivo(e2) - getTiempoInactivo(e1));
    const float TIEMPO_TOTAL = TIEMPO_ACTIVO + TIEMPO_INACTIVO;

    float usoCPU = 1.f * TIEMPO_ACTIVO / TIEMPO_TOTAL;
    salida[i - 1] = CpuMon::obtenerBarra(&usoCPU, e1.cpu);
  }

  ui::mostrarVentana(ventanaMonitorCPU, &salida, &columnas, &num_sectores,
                     &txt::g_monitorTitulo);
}

std::string CpuMon::obtenerBarra(float *porcentaje, std::string cpu) {
  std::string barra = " CPU ";

  if (CpuMon::num_hilos > 9 && cpu.length() < 2) {
    barra.append("0").append(cpu).append(" [");
  } else {
    barra.append(cpu).append(" [");
  }
  std::string cierre = "] ";

  int ent_por = (int)(100 * *porcentaje);

  if (ent_por < 10) {
    cierre.append("  ");
  } else if (ent_por < 100) {
    cierre.append(" ");
  }

  cierre.append(std::to_string(ent_por));
  cierre.append("%%");

  // El ancho se calcula con el ancho máximo menos la longitud de los caracteres
  // iniciales y menos los carateres de cierre.
  int ancho = CpuMon::columnas - barra.length() - cierre.length() - 3;

  // Bucle para almohadillas
  int num_veces = (int)(ancho * *porcentaje);
  for (uint16_t i = 0; i < num_veces; i++) {
    barra.append("#");
  }

  // Bucle para guiones
  num_veces = ancho - num_veces;
  for (uint16_t i = 0; i < num_veces; i++) {
    barra.append("-");
  }

  barra.append(cierre);
  return barra;
}
